@@ -3,6 +3,7 @@ from lingtrain_aligner import helper, preprocessor
 import json
 import pathlib
 import copy
+from operator import itemgetter
 
 H1_MARK = preprocessor.PARAGRAPH_MARK + preprocessor.H1
 H2_MARK = preprocessor.PARAGRAPH_MARK + preprocessor.H2
@@ -25,7 +26,7 @@ def is_empty_cells(db_path):
     return False
     
 
-def get_paragraphs(db_path, direction="from"):
+def get_paragraphs(db_path, direction="from", par_amount=0):
     """Read all paragraphs with marks from database"""
     # default direction is 'from'
     if direction != "to":
@@ -63,8 +64,13 @@ def get_paragraphs(db_path, direction="from"):
 
     gen_main = get_next_paragraph(
         index, data, paragraphs_from_dict, paragraphs_to_dict, direction)
-    par_info = [(par_from, par_to, par_id)
-                for par_from, par_to, par_id, _, _ in gen_main]
+
+    par_info, count = [], 0
+    for par_from, par_to, par_id, _, _ in gen_main:
+        par_info.append((par_from, par_to, par_id))
+        count += 1
+        if par_amount != 0 and count == par_amount:
+            break
     par_info_list = list(zip(*par_info))
 
     paragraphs_from, paragraphs_to, par_ids = par_info_list[0], par_info_list[1], par_info_list[2]
@@ -412,7 +418,13 @@ def get_next_paragraph(index, data, paragraphs_from_dict, paragraphs_to_dict, di
     yield curr_from, curr_to, prev_paragraph_max, curr_splitted_ids_from, curr_splitted_ids_to
 
 
-def create_book(lang_ordered, paragraphs, delimeters, metas, sent_counter, output_path, template, styles=[]):
+def sort_meta(metas):
+    for lang in metas["items"]:
+        for mark in metas["items"][lang]:
+            metas["items"][lang][mark].sort(key=lambda x: x[2])
+
+
+def create_book(lang_ordered, paragraphs, delimeters, metas, sent_counter, output_path, template, styles=[], highlight="through"):
     """Generate html"""
     # ensure path is existed
     pathlib.Path(output_path).parent.mkdir(parents=True, exist_ok=True)
@@ -427,6 +439,8 @@ def create_book(lang_ordered, paragraphs, delimeters, metas, sent_counter, outpu
         css = generate_css([])
         sent_cycle = 2
     
+    meta = sort_meta(metas)
+
     min_par_len = min([len(paragraphs[x]) for x in paragraphs])
     min_par_len = min(min_par_len, len(delimeters))
 
@@ -461,14 +475,20 @@ def create_book(lang_ordered, paragraphs, delimeters, metas, sent_counter, outpu
         res_html.write("</div>")
 
         #--------------------TITLE and AUTHOR
-        res_html.write("<div class='dt-row header'>")
+        res_html.write("<div class='dt-row header title-cell'>")
         for lang in lang_ordered:
-            res_html.write("<div class='par dt-cell'>")
+            res_html.write("<div class='title-cell dt-cell'>")
             meta = metas["items"][lang]
             title = get_meta(meta, preprocessor.TITLE)
-            author = get_meta(meta, preprocessor.AUTHOR)
             if title:
                 res_html.write("<h1 class='lt-title'>" + title + "</h1>")
+            res_html.write("</div>")
+        res_html.write("</div>")
+        res_html.write("<div class='dt-row header'>")
+        for lang in lang_ordered:
+            res_html.write("<div class='author-cell dt-cell'>")
+            meta = metas["items"][lang]
+            author = get_meta(meta, preprocessor.AUTHOR)
             if author:
                 res_html.write("<h1 class='lt-author'>" + author + "</h1>")
             res_html.write("</div>")
@@ -476,24 +496,25 @@ def create_book(lang_ordered, paragraphs, delimeters, metas, sent_counter, outpu
 
         next_mark, next_meta_par_id = get_next_meta_par_id(metas)
 
+        j = 0
         for actual_paragraphs_id in range(min_par_len):
             real_par_id = delimeters[actual_paragraphs_id]
 
             while next_meta_par_id <= real_par_id:
                 _ = write_next_polyheader(res_html, next_mark, metas, lang_ordered)
                 next_mark, next_meta_par_id = get_next_meta_par_id(metas)
-
+            
             res_html.write("<div class='dt-row'>")
             for lang in lang_ordered:
-
-                res_html.write(f"<div class='par dt-cell'><div class='book-par-id'>{real_par_id}</div>")
-
+                
+                res_html.write(f"<div class='par dt-cell'><div class='book-par-id'>{real_par_id + 1}</div>")
                 for k, sent in enumerate(paragraphs[lang][actual_paragraphs_id]):
+                    sent_cycle_index = (j+k) % sent_cycle if highlight == "through" else k % sent_cycle
                     res_html.write(
-                        f"<span class='s s{k%sent_cycle}'>{sent}</span>")
-
+                        f"<span class='s s{sent_cycle_index%sent_cycle}'>{sent}</span>")
                 res_html.write("</div>")
 
+            j += len(paragraphs[lang][actual_paragraphs_id])
             res_html.write("</div>")
 
         while next_mark:
@@ -505,7 +526,7 @@ def create_book(lang_ordered, paragraphs, delimeters, metas, sent_counter, outpu
         res_html.write("</body></html>")
 
 
-def create_polybook(lang_ordered, paragraphs, delimeters, metas, output_path, template, styles=[]):
+def create_polybook(lang_ordered, paragraphs, delimeters, metas, output_path, template, styles=[], highlight="through"):
     """Generate multilingual html"""
     # ensure path is existed
     pathlib.Path(output_path).parent.mkdir(parents=True, exist_ok=True)
@@ -521,10 +542,13 @@ def create_polybook(lang_ordered, paragraphs, delimeters, metas, output_path, te
         css = generate_css([], cols=langs_count)
         sent_cycle = 2
 
+    meta = sort_meta(metas)
+
     with open(output_path, "w", encoding="utf8") as res_html:
         # --------------------HEAD
         res_html.write(f"""
 <html><head>
+    <meta charset="utf-8">
     <link rel="preconnect" href="https://fonts.gstatic.com">
     <link href="https://fonts.googleapis.com/css2?family=Noto+Serif:wght@400&display=swap" rel="stylesheet">
     <link href="https://fonts.googleapis.com/css2?family=Oswald:wght@300;400&display=swap" rel="stylesheet">
@@ -546,14 +570,20 @@ def create_polybook(lang_ordered, paragraphs, delimeters, metas, output_path, te
         res_html.write("</div>")
 
         #--------------------TITLE and AUTHOR
-        res_html.write("<div class='dt-row header'>")
+        res_html.write("<div class='dt-row header title-cell'>")
         for lang in lang_ordered:
-            res_html.write("<div class='par dt-cell'>")
+            res_html.write("<div class='title-cell dt-cell'>")
             meta = metas["items"][lang]
             title = get_meta(meta, preprocessor.TITLE)
-            author = get_meta(meta, preprocessor.AUTHOR)
             if title:
                 res_html.write("<h1 class='lt-title'>" + title + "</h1>")
+            res_html.write("</div>")
+        res_html.write("</div>")
+        res_html.write("<div class='dt-row header'>")
+        for lang in lang_ordered:
+            res_html.write("<div class='author-cell dt-cell'>")
+            meta = metas["items"][lang]
+            author = get_meta(meta, preprocessor.AUTHOR)
             if author:
                 res_html.write("<h1 class='lt-author'>" + author + "</h1>")
             res_html.write("</div>")
@@ -562,7 +592,8 @@ def create_polybook(lang_ordered, paragraphs, delimeters, metas, output_path, te
         # --------------------PARAGRAPHS
         next_mark, next_meta_par_id = get_next_meta_par_id(metas)
         min_par_len = min([len(paragraphs[x]) for x in paragraphs])
-
+        
+        j = 0
         for actual_paragraphs_id in range(min_par_len):
             real_par_id = delimeters[actual_paragraphs_id]
 
@@ -572,15 +603,16 @@ def create_polybook(lang_ordered, paragraphs, delimeters, metas, output_path, te
 
             res_html.write("<div class='dt-row'>")
             for lang in lang_ordered:
-
+                
                 res_html.write(f"<div class='par dt-cell'>  ")
 
                 for k, sent in enumerate(paragraphs[lang][actual_paragraphs_id]):
+                    sent_cycle_index = (j+k) % sent_cycle if highlight == "through" else k % sent_cycle
                     res_html.write(
-                        f"<span class='s s{k%sent_cycle}'>{sent}</span>")
-
+                        f"<span class='s s{sent_cycle_index%sent_cycle}'>{sent}</span>")
                 res_html.write("</div>")
 
+            j += len(paragraphs[lang][actual_paragraphs_id])
             res_html.write("</div>")
 
         while next_mark:
@@ -592,7 +624,7 @@ def create_polybook(lang_ordered, paragraphs, delimeters, metas, output_path, te
         res_html.write("</body></html>")
 
 
-def create_polybook_preview(lang_ordered, paragraphs, delimeters, metas, template, styles=[], par_amount=0):
+def create_polybook_preview(lang_ordered, paragraphs, delimeters, metas, template, styles=[], par_amount=0, highlight="through"):
     """Generate multiligual html preview"""
     # ensure path is existed
     langs_count = len(lang_ordered)
@@ -608,6 +640,8 @@ def create_polybook_preview(lang_ordered, paragraphs, delimeters, metas, templat
         template = "simple"
         sent_cycle = 4
 
+    meta = sort_meta(metas)
+
     res_html = ""
     # --------------------BOOK
     res_html += "<div class='dt cont'>"
@@ -620,14 +654,20 @@ def create_polybook_preview(lang_ordered, paragraphs, delimeters, metas, templat
     res_html += "</div>"
 
     #--------------------TITLE and AUTHOR
-    res_html += "<div class='dt-row header'>"
+    res_html += "<div class='dt-row header title-cell'>"
     for lang in lang_ordered:
-        res_html += "<div class='par dt-cell'>"
+        res_html += "<div class='title-cell dt-cell'>"
         meta = metas["items"][lang]
         title = get_meta(meta, preprocessor.TITLE)
-        author = get_meta(meta, preprocessor.AUTHOR)
         if title:
             res_html += "<h1 class='lt-title'>" + title + "</h1>"
+        res_html += "</div>"
+    res_html += "</div>"
+    res_html += "<div class='dt-row header'>"
+    for lang in lang_ordered:
+        res_html += "<div class='author-cell dt-cell'>"
+        meta = metas["items"][lang]
+        author = get_meta(meta, preprocessor.AUTHOR)
         if author:
             res_html += "<h1 class='lt-author'>" + author + "</h1>"
         res_html += "</div>"
@@ -639,6 +679,7 @@ def create_polybook_preview(lang_ordered, paragraphs, delimeters, metas, templat
     if par_amount > 0:
         min_par_len = min(min_par_len, par_amount)
 
+    j = 0
     for actual_paragraphs_id in range(min_par_len):
         real_par_id = delimeters[actual_paragraphs_id]
 
@@ -648,14 +689,14 @@ def create_polybook_preview(lang_ordered, paragraphs, delimeters, metas, templat
 
         res_html += "<div class='dt-row'>"
         for lang in lang_ordered:
-
-            res_html += f"<div class='par dt-cell'><div class='book-par-id'>{real_par_id}</div>"
+            res_html += f"<div class='par dt-cell'><div class='book-par-id'>{real_par_id + 1}</div>"
 
             for k, sent in enumerate(paragraphs[lang][actual_paragraphs_id]):
-                res_html += f"<span class='s s{k%sent_cycle} {template}'>{sent}</span>"
+                sent_cycle_index = (j+k) % sent_cycle if highlight == "through" else k % sent_cycle
+                res_html += f"<span class='s s{sent_cycle_index} {template}'>{sent}</span>"
 
             res_html += "</div>"
-
+        j += len(paragraphs[lang][actual_paragraphs_id])
         res_html += "</div>"
 
     # --------------------END BOOK
@@ -668,11 +709,14 @@ def get_next_meta_par_id(metas):
     meta = metas["items"][lang_code]
     min_par_id = float('Inf')
     next_mark = ""
+
     for mark in MARKS:
         if mark in meta and meta[mark]:
-            if meta[mark][0][2] < min_par_id:
+            gen = (x for x in meta[mark])
+            min_par_mark = min(gen, key=itemgetter(2))
+            if min_par_mark[2] < min_par_id:
                 next_mark = mark
-                min_par_id = meta[mark][0][2]
+                min_par_id = min_par_mark[2]
     return next_mark, min_par_id
 
 
@@ -704,7 +748,7 @@ def write_next_polyheader(writer, next_mark, metas_dict, lang_ordered, add_strin
         elif next_mark == preprocessor.QUOTE_NAME:
             el = f"<div class='par dt-cell'><div class='lt-quote lt-quote-name'>{val[0]}</div></div>"
         elif next_mark == preprocessor.IMAGE:
-            el = f"<div class='par dt-cell text-center'><img class='lt-image' src='{val[0]}'/></div>"
+            el = f"<div class='par dt-cell text-center'><div class='img-cont'><div class='lt-image-mask'></div><img class='lt-image' src='{val[0]}'/></div></div>"
         else:
             el = f"<div class='par dt-cell'><{HEADER_HTML_MAPPING[next_mark]}>{val[0]}</{HEADER_HTML_MAPPING[next_mark]}></div>"
 
@@ -779,7 +823,9 @@ HEADER_HTML_MAPPING = {
 }
 
 # DIVIDER_URL = "https://habrastorage.org/webt/nr/av/qa/nravqa-wy0sg8kgwr3cfli8veym.png"
-DIVIDER_URL = "https://habrastorage.org/webt/q9/1t/cy/q91tcypgjnrrsmrfcviyzzdvfsk.png"
+# DIVIDER_URL = "https://habrastorage.org/webt/q9/1t/cy/q91tcypgjnrrsmrfcviyzzdvfsk.png"
+DIVIDER_URL = "https://habrastorage.org/webt/28/2n/1b/282n1b8oxclxp-jacqfc3sytbqm.png"
+
 
 HTML_FOOTER = """
 <div class="lt-footer">🚀 created in lingtrain alignment studio</div>"""
@@ -888,8 +934,8 @@ h3 {
 }
 
 .divider-img {
-    width: 50px;
-    height: 50px;
+    # width: 50px;
+    # height: 50px;
 }
 
 .flag-img {
@@ -922,16 +968,34 @@ h3 {
 .lt-image {
     max-height: 300px;
     max-width: 800px;
+    -webkit-mask-image: url(https://hsto.org/webt/wg/wv/ai/wgwvai84o5fvgpok4mnannr7jca.png);
+    mask-image: url(https://hsto.org/webt/wg/wv/ai/wgwvai84o5fvgpok4mnannr7jca.png);
+    -webkit-mask-repeat: no-repeat;
+    mask-repeat: no-repeat;
+    -webkit-mask-size: contain;
+    mask-size: contain;
+    -webkit-mask-position: center;
+    mask-position: center;
+    margin: 20px 0;
 }
 .lt-title {
     font-size: 52px;
     text-align: center;
 }
 .lt-author {
+    margin-top: -10px;
     font-size: 40px;
     text-align: center;
 }
 
+.title-cell {
+    padding: 20px 20px 0px 20px;
+}
+
+.author-cell {
+    padding: 0px 0px 30px 0px;
+}
+
 @media print {
     .lt-title {
         font-size: 32px;
